﻿// game.js for Perlenspiel 2.1

/*
Perlenspiel is Copyright © 2009-12 Worcester Polytechnic Institute.
This file is part of Perlenspiel.

Perlenspiel is free software: you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published
by the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Perlenspiel is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You may have received a copy of the GNU Lesser General Public License
along with Perlenspiel. If not, see <http://www.gnu.org/licenses/>.
*/

// This is a template for creating new Perlenspiel games
// All of the functions below MUST exist, or the engine will stop and complain!

// The following comment line is for JSLint. Don't remove it!

/*global PS */

var gridSize = 32;
var running = false;
var painting = false;
var instructions = true;

// PS.Init ()
// Initializes the game
// This function normally includes a call to PS.GridSize (x, y)
// where x and y are the desired dimensions of the grid

PS.Init = function () {
    "use strict";

    // change to the dimensions you want

    PS.GridSize(gridSize, gridSize);
    PS.BeadBorderWidth(PS.ALL, PS.ALL, 0);
    PS.StatusText("Conway's Game of Life");
    PS.BeadFlash(PS.ALL, PS.ALL, false);

    for (var i = 0; i < gridSize; i++) {
        for (var j = 0; j < gridSize; j++) {
            PS.BeadColor(i, j, PS.COLOR_WHITE);
            PS.BeadData(i, j, false);
        }
    }

    PrintInstructions();

    //start clock
    PS.Clock(10);
};

// PS.Click (x, y, data)
// This function is called whenever a bead is clicked
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set

PS.Click = function (x, y, data) {
    "use strict";

    if (instructions) return Begin();
    if (running) return;

    painting = true;

    Paint(x, y);
    // put code here for bead clicks
};

// PS.Release (x, y, data)
// This function is called whenever a mouse button is released over a bead
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set

PS.Release = function (x, y, data) {
    "use strict";

    if (running) return;
    painting = false;
    // Put code here for when the mouse button is released over a bead	
};

// PS.Enter (x, y, button, data)
// This function is called whenever the mouse moves over a bead
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set

PS.Enter = function (x, y, data) {
    "use strict";

    if (running) return;

    if (!painting) return;

    Paint(x, y);

    // Put code here for when the mouse enters a bead	
};

// PS.Leave (x, y, data)
// This function is called whenever the mouse moves away from a bead
// It doesn't have to do anything
// x = the x-position of the bead on the grid
// y = the y-position of the bead on the grid
// data = the data value associated with this bead, 0 if none has been set

PS.Leave = function (x, y, data)
{
	"use strict";

	// Put code here for when the mouse leaves a bead	
};

// PS.KeyDown (key, shift, ctrl)
// This function is called whenever a key on the keyboard is pressed
// It doesn't have to do anything
// key = the ASCII code of the pressed key, or one of the following constants:
// Arrow keys = PS.ARROW_UP, PS.ARROW_DOWN, PS.ARROW_LEFT, PS.ARROW_RIGHT
// Function keys = PS.F1 through PS.F1
// shift = true if shift key is held down, false otherwise
// ctrl = true if control key is held down, false otherwise

PS.KeyDown = function (key, shift, ctrl) {
    "use strict";

    if (instructions) return Begin();
    if (key == 32) {
        running = !running;
    }

    // Put code here for when a key is pressed	
};

// PS.KeyUp (key, shift, ctrl)
// This function is called whenever a key on the keyboard is released
// It doesn't have to do anything
// key = the ASCII code of the pressed key, or one of the following constants:
// Arrow keys = PS.ARROW_UP, PS.ARROW_DOWN, PS.ARROW_LEFT, PS.ARROW_RIGHT
// Function keys = PS.F1 through PS.F12
// shift = true if shift key is held down, false otherwise
// ctrl = true if control key is held down, false otherwise

PS.KeyUp = function (key, shift, ctrl)
{
	"use strict";
	
	// Put code here for when a key is released	
};

// PS.Wheel (dir)
// This function is called whenever the mouse wheel moves forward or backward
// It doesn't have to do anything
// dir = 1 if mouse wheel moves forward, -1 if backward

PS.Wheel = function (dir)
{
	"use strict";

	// Put code here for when a key is pressed	
};

// PS.Tick ()
// This function is called on every clock tick
// if a timer has been activated with a call to PS.Timer()
// It doesn't have to do anything

PS.Tick = function () {
    "use strict";

    if (running) {

        //iterate through gride
        for (var i = 0; i < gridSize; i++) {
            for (var j = 0; j < gridSize; j++) {

                var alive = PS.BeadData(i, j);
                var numAliveNeighbors = 0;

                for (var i2 = -1; i2 <= 1; i2++) {
                    for (var j2 = -1; j2 <= 1; j2++) {

                        if (i2 == 0 && j2 == 0) continue;
                        if (i + i2 < 0) continue;
                        if (j + j2 < 0) continue;
                        if (i + i2 >= gridSize) continue;
                        if (j + j2 >= gridSize) continue;

                        if (PS.BeadColor(i + i2, j + j2) == PS.COLOR_BLUE)
                            numAliveNeighbors += 1;
                        
                    }
                }


                if (!alive && numAliveNeighbors == 3)
                    PS.BeadData(i, j, true);
                else if (alive && (numAliveNeighbors > 3 || numAliveNeighbors < 2))
                    PS.BeadData(i, j, false);
            }
        }

        //update visuals
        for (var i = 0; i < gridSize; i++) {
            for (var j = 0; j < gridSize; j++) {

                if (PS.BeadData(i, j))
                    PS.BeadColor(i, j, PS.COLOR_BLUE);
                else
                    PS.BeadColor(i, j, PS.COLOR_WHITE);
            }
        }


    }
    // Put code here to handle clock ticks
};

Paint = function (x, y) {

    if (PS.BeadColor(x, y) == PS.COLOR_WHITE) {
        PS.BeadColor(x, y, PS.COLOR_BLUE);
        PS.BeadData(x, y, true);
    }
    else {
        PS.BeadColor(x, y, PS.COLOR_WHITE);
        PS.BeadData(x, y, false);
    }

};

PrintInstructions = function () {
    PS.BeadGlyphColor(PS.ALL, PS.ALL, PS.COLOR_RED);
    PS.BeadGlyph(0, 1, 'P');
    PS.BeadGlyph(1, 1, 'a');
    PS.BeadGlyph(2, 1, 'i');
    PS.BeadGlyph(3, 1, 'n');
    PS.BeadGlyph(4, 1, 't');
    PS.BeadGlyph(5, 1, ' ');
    PS.BeadGlyph(6, 1, 'l');
    PS.BeadGlyph(7, 1, 'i');
    PS.BeadGlyph(8, 1, 'v');
    PS.BeadGlyph(9, 1, 'e');
    PS.BeadGlyph(10, 1, ' ');
    PS.BeadGlyph(11, 1, 'c');
    PS.BeadGlyph(12, 1, 'e');
    PS.BeadGlyph(13, 1, 'l');
    PS.BeadGlyph(14, 1, 'l');
    PS.BeadGlyph(15, 1, 's');
    PS.BeadGlyph(16, 1, ' ');
    PS.BeadGlyph(17, 1, 'w');
    PS.BeadGlyph(18, 1, 'i');
    PS.BeadGlyph(19, 1, 't');
    PS.BeadGlyph(20, 1, 'h');
    PS.BeadGlyph(21, 1, ' ');
    PS.BeadGlyph(22, 1, 't');
    PS.BeadGlyph(23, 1, 'h');
    PS.BeadGlyph(24, 1, 'e');
    PS.BeadGlyph(25, 1, ' ');
    PS.BeadGlyph(26, 1, 'm');
    PS.BeadGlyph(27, 1, 'o');
    PS.BeadGlyph(28, 1, 'u');
    PS.BeadGlyph(29, 1, 's');
    PS.BeadGlyph(30, 1, 'e');

    PS.BeadGlyph(0, 3, 'S');
    PS.BeadGlyph(1, 3, 'p');
    PS.BeadGlyph(2, 3, 'a');
    PS.BeadGlyph(3, 3, 'c');
    PS.BeadGlyph(4, 3, 'e');
    PS.BeadGlyph(5, 3, ' ');
    PS.BeadGlyph(6, 3, 'b');
    PS.BeadGlyph(7, 3, 'a');
    PS.BeadGlyph(8, 3, 'r');
    PS.BeadGlyph(9, 3, ' ');
    PS.BeadGlyph(10, 3, 't');
    PS.BeadGlyph(11, 3, 'o');
    PS.BeadGlyph(12, 3, ' ');
    PS.BeadGlyph(13, 3, 's');
    PS.BeadGlyph(14, 3, 't');
    PS.BeadGlyph(15, 3, 'a');
    PS.BeadGlyph(16, 3, 'r');
    PS.BeadGlyph(17, 3, 't');
    PS.BeadGlyph(18, 3, '/');
    PS.BeadGlyph(19, 3, 's');
    PS.BeadGlyph(20, 3, 't');
    PS.BeadGlyph(21, 3, 'o');
    PS.BeadGlyph(22, 3, 'p');
    PS.BeadGlyph(23, 3, ' ');
    PS.BeadGlyph(24, 3, 't');
    PS.BeadGlyph(25, 3, 'h');
    PS.BeadGlyph(26, 3, 'e');
    PS.BeadGlyph(27, 3, ' ');
    PS.BeadGlyph(28, 3, 'g');
    PS.BeadGlyph(29, 3, 'a');
    PS.BeadGlyph(30, 3, 'm');
    PS.BeadGlyph(31, 3, 'e');

    PS.BeadGlyph(0, 10, ' ');
    PS.BeadGlyph(1, 10, ' ');
    PS.BeadGlyph(2, 10, 'P');
    PS.BeadGlyph(3, 10, 'r');
    PS.BeadGlyph(4, 10, 'e');
    PS.BeadGlyph(5, 10, 's');
    PS.BeadGlyph(6, 10, 's');
    PS.BeadGlyph(7, 10, ' ');
    PS.BeadGlyph(8, 10, 'a');
    PS.BeadGlyph(9, 10, 'n');
    PS.BeadGlyph(10, 10, 'y');
    PS.BeadGlyph(11, 10, ' ');
    PS.BeadGlyph(12, 10, 'k');
    PS.BeadGlyph(13, 10, 'e');
    PS.BeadGlyph(14, 10, 'y');
    PS.BeadGlyph(15, 10, ' ');
    PS.BeadGlyph(16, 10, 't');
    PS.BeadGlyph(17, 10, 'o');
    PS.BeadGlyph(18, 10, ' ');
    PS.BeadGlyph(19, 10, 'c');
    PS.BeadGlyph(20, 10, 'o');
    PS.BeadGlyph(21, 10, 'n');
    PS.BeadGlyph(22, 10, 't');
    PS.BeadGlyph(23, 10, 'i');
    PS.BeadGlyph(24, 10, 'n');
    PS.BeadGlyph(25, 10, 'u');
    PS.BeadGlyph(26, 10, 'e');
    PS.BeadGlyph(27, 10, ' ');
    PS.BeadGlyph(28, 10, ' ');
    PS.BeadGlyph(29, 10, ' ');
    PS.BeadGlyph(30, 10, ' ');
    PS.BeadGlyph(31, 10, ' ');
};

function Begin() {
    PS.BeadBorderWidth(PS.ALL, PS.ALL, 1);
    instructions = false;

    for (var i = 0; i < gridSize; i++)
        for (var j = 0; j < gridSize; j++)
            PS.BeadGlyph(i, j, ' ');
};